"""Constants for the op model."""

RESPONSE = "response"
ERROR = "error"
ERROR_LIST = "error_list"
ERROR_CONTEXT = "error_context"
MAJORITY_VOTE = "majority_vote"
AVERAGE_SCORE = "average_score"
VOTES = "votes"
SCORES = "scores"
SAMPLES = "samples"


MAX_ATTEMPTS = 3
